@@ -8,6 +8,7 @@ FUNCTIONS_HPP = '''
 #include <string>
 #include <algorithm>
 #include "config.h"
+#include <assert.h>
 
 #include "pugixml/pugixml.hpp"
 #include "jsoncpp/json.h"
@@ -33,6 +34,13 @@ namespace @{namespace}
         list.push_back(t);
     }
 
+    template <class T, class I>
+    void list_insert(std::vector<T>& list, size_t index, const I& t)
+    {
+        assert(index <= list.size());
+        list.insert(list.begin() + index, t);
+    }
+
     template <class T, class I>
     void list_remove(std::vector<T>& list, const I& t)
     {
@@ -41,6 +49,13 @@ namespace @{namespace}
             list.erase(iter);
     }
 
+    template <class T>
+    void list_erase(std::vector<T>& list, size_t index)
+    {
+        assert(index < list.size());
+        list.erase(list.begin() + index);
+    }
+
     template <class T>
     int list_size(const std::vector<T>& vector)
     {
@@ -53,11 +68,31 @@ namespace @{namespace}
         vector.clear();
     }
 
+    template <class T>
+    void list_resize(std::vector<T>& vector, int size)
+    {
+        vector.resize(size);
+    }
+
     template <class T, class P>
     int map_size(const std::map<T, P>& map)
     {
         return static_cast<int>(map.size());
     }
+    template <class T, class P>
+    void map_clear(std::map<T, P>& map)
+    {
+        map.clear();
+    }
+    template <class T, class P>
+    void map_remove(std::map<T, P>& map, const T& key)
+    {
+        auto iter = map.find(key);
+        if(iter != map.end())
+        {
+            map.erase(iter);
+        }
+    }
 
     bool string_empty(const std::string& string);
     int string_size(const std::string& string);
@@ -475,12 +510,8 @@ namespace @{namespace}
     inline intrusive_ptr<Type> make_intrusive(TArgs&& ... _Args)
     {
         intrusive_ptr<Type> holder;
-        Type* ptr = new Type(std::forward<TArgs>(_Args)...);
-        if(ptr)
-        {
-            holder.reset(ptr);
-            ptr->release();
-        }
+        holder.reset(new Type(std::forward<TArgs>(_Args)...));
+        holder->release();
         return holder;
     }
 
@@ -499,17 +530,7 @@ FACTORY_HPP = '''#ifndef __@{namespace}_Factory_h__
 #include "jsoncpp/json.h"
 #include <sstream>
 #include "pugixml/pugixml.hpp"
-
-#define REGISTRATION_OBJECT(TType)                                      \\
-class registration__##TType                                             \\
-{                                                                       \\
-public:                                                                 \\
-    registration__##TType()                                             \\
-    {                                                                   \\
-        Factory::shared().registrationCommand<TType>( TType::TYPE );    \\
-    }                                                                   \\
-} ___registration___##TType;
-
+@{registration}
 namespace @{namespace}
 {
     
@@ -571,6 +592,7 @@ namespace @{namespace}
             result->release();
             return result;
         }
+        
         {{format=json}}
         template <class TType>
         static std::string serialize_command_to_json(intrusive_ptr<TType> command)
@@ -596,7 +618,16 @@ namespace @{namespace}
             command->deserialize_json(json[type]);
             return command;
         }
-        {{end_format=json}}{{format=xml}}
+        template <class TType>
+        static intrusive_ptr<TType> clone_object(intrusive_ptr<TType> object)
+        {
+            auto payload = Factory::serialize_command_to_json<TType>(object);
+            auto clone = Factory::create_command_from_json<TType>(payload);
+            return clone;
+        }
+        {{end_format=json}}
+        
+        {{format=xml}}
         template <class TType>
         static std::string serialize_command_to_xml(intrusive_ptr<TType> command)
         {
@@ -630,7 +661,84 @@ namespace @{namespace}
             command->deserialize_xml(root);
             return command;
         }
+
+        template <class TType>
+        static intrusive_ptr<TType> clone_object(intrusive_ptr<TType> object)
+        {
+            auto payload = Factory::serialize_command_to_xml<TType>(object);
+            auto clone = Factory::create_command_from_xml<TType>(payload);
+            return clone;
+        }
         {{end_format=xml}}
+        
+        {{format=both}}
+        template <class TType>
+        static std::string serialize_command_to_xml(intrusive_ptr<TType> command)
+        {
+            pugi::xml_document doc;
+            auto root = doc.append_child(command->get_type().c_str());
+            command->serialize_xml(root);
+            
+            std::stringstream stream;
+            pugi::xml_writer_stream writer(stream);
+            #ifdef NDEBUG
+            doc.save(writer,
+                     "",
+                     pugi::format_no_declaration | pugi::format_raw,
+                     pugi::xml_encoding::encoding_utf8);
+            #else
+            doc.save(writer,
+                     PUGIXML_TEXT(" "),
+                     pugi::format_no_declaration | pugi::format_indent,
+                     pugi::xml_encoding::encoding_utf8);
+            #endif
+            return stream.str();
+        }
+        
+        template <class TType>
+        static intrusive_ptr<TType> create_command_from_xml(const std::string& payload)
+        {
+            pugi::xml_document doc;
+            doc.load(payload.c_str());
+            auto root = doc.root().first_child();
+            auto command = shared().build<TType>(root.name());
+            command->deserialize_xml(root);
+            return command;
+        }
+        
+        template <class TType>
+        static std::string serialize_command_to_json(intrusive_ptr<TType> command)
+        {
+            Json::Value json;
+            command->serialize_json(json[command->get_type()]);
+            
+            Json::StreamWriterBuilder wbuilder;
+            wbuilder["indentation"] = "";
+            return Json::writeString(wbuilder, json);
+        }
+        
+        template <class TType>
+        static intrusive_ptr<TType> create_command_from_json(const std::string& payload)
+        {
+            Json::Value json;
+            Json::Reader reader;
+            reader.parse(payload, json);
+            
+            auto type = json.getMemberNames()[0];
+            auto command = shared().build<TType>(type);
+            if (command != nullptr)
+            command->deserialize_json(json[type]);
+            return command;
+        }
+        
+        template <class TType>
+        static intrusive_ptr<TType> clone_object(intrusive_ptr<TType> object)
+        {
+            auto payload = Factory::serialize_command_to_json<TType>(object);
+            auto clone = Factory::create_command_from_json<TType>(payload);
+            return clone;
+        }
+        {{end_format=both}}
     private:
         std::map<std::string, IBuilder*> _builders;
     };
@@ -638,7 +746,17 @@ namespace @{namespace}
 
 #endif // __@{namespace}_Factory_h__
 '''
-
+FACTORY_REGISTRATION = '''
+#define REGISTRATION_OBJECT(TType)                                      \\
+class registration__##TType                                             \\
+{                                                                       \\
+public:                                                                 \\
+    registration__##TType()                                             \\
+    {                                                                   \\
+        Factory::shared().registrationCommand<TType>(TType::TYPE);      \\
+    }                                                                   \\
+} ___registration___##TType;
+'''
 
 FILES_DICT = [
     ['@{namespace}_extensions.h', FUNCTIONS_HPP],
